@@ -9,6 +9,7 @@ import http.cookiejar
 import json
 import os
 import sqlite3
+import yaml
 import sys
 import tempfile
 import urllib.parse
@@ -17,14 +18,15 @@ from pathlib import Path
 
 import keyring
 import lz4.block
-import yaml
 from Cryptodome.Cipher import AES
 from Cryptodome.Protocol.KDF import PBKDF2
 from pydantic import BaseModel
 from pydantic.dataclasses import dataclass
+from requests.auth import HTTPBasicAuth
 
 from bigeye_sdk.functions.aws import get_secret
 from bigeye_sdk.authentication.enums import BrowserType, OperatingSystem
+from bigeye_sdk.functions.file_functs import create_subdir_if_not_exists
 from bigeye_sdk.exceptions import BrowserAuthException
 from bigeye_sdk.log import get_logger
 from bigeye_sdk.serializable import YamlSerializable
@@ -32,6 +34,9 @@ from bigeye_sdk.serializable import YamlSerializable
 log = get_logger(__file__)
 
 HOME_DIR = str(Path.home())
+DEFAULT_CRED_FILE = os.path.join(HOME_DIR, '.bigeye', 'credentials.ini')
+OLD_DEFAULT_CRED_FILE = os.path.join(HOME_DIR, '.bigeye', 'default_cred.json')
+
 
 
 class AwsSecretIdentifier(BaseModel):
@@ -40,36 +45,92 @@ class AwsSecretIdentifier(BaseModel):
 
 
 @dataclass
-class ApiAuthConf(YamlSerializable):
+class ApiAuth(YamlSerializable):
+
+    @abc.abstractmethod
+    def get_auth_headers(self):
+        pass
 
     @classmethod
-    def load_from_file(cls, file: str) -> ApiAuthConf:
+    def load(cls, auth_file: str, workspace: str = 'DEFAULT') -> ApiAuth:
+        auth_file = cls.find_user_credentials(auth_file)
+        if os.path.isfile(auth_file):
+            if auth_file.lower().endswith('.json'):
+                return cls.load_from_json_file(auth_file)
+            else:
+                return cls.load_from_ini_file(auth_file,workspace)
+        else:
+            return cls.load_from_base64(auth_file)
+
+    @classmethod
+    def load_from_ini_file(cls, auth_file: str, workspace: str) -> ApiAuth:
+        log.info(f'Loading API Conf: {auth_file}')
+        cp = configparser.ConfigParser(default_section=None, interpolation=None)
+        cp.read(auth_file)
+        # Convert browser type to proper enum (config parser reads everything as strings)
+        for key, value in cp[workspace].items():
+            if key == 'browser':
+                cp[workspace][key] = BrowserType[value]
+        return cls.factory(cp[workspace])
+
+    @classmethod
+    def load_from_json_file(cls, file: str) -> ApiAuth:
         log.info(f'Loading API Conf: {file}')
         with open(file) as json_file:
             return cls.factory(yaml.safe_load(json_file))
 
     @classmethod
     def load_cred_from_aws_secret(cls, region_name: str, secret_name: str):
-        log.info(f'Loading Snowflake Credential Secret.')
-        return ApiAuthConf.factory(get_secret(region_name=region_name,
+        log.info(f'Loading AWS Secret: {secret_name}')
+        return cls.factory(get_secret(region_name=region_name,
                                               secret_name=secret_name))
 
     @classmethod
     def load_from_base64(cls, encoded: str):
         try:
-            log.info("Loading API conf")
+            log.info("Loading API Conf")
             conf_dict = json.loads(base64.b64decode(encoded).decode('utf-8'))
             return cls.factory(conf_dict)
         except binascii.Error as e:
             log.error('Error decoding API configuration. Verify it is valid base64.')
             log.error(f'Error message: {e}')
             raise e
+        
+    @classmethod
+    def save_as_file(cls, auth_filename: str, cred: BrowserAPIAuth | BasicAPIAuth, workspace: str) -> None:
+        cp = configparser.ConfigParser(default_section=None, interpolation=None)
+        if not auth_filename:
+            auth_filename = cls.find_user_credentials(auth_filename)
+        cp.read(auth_filename)
+        cp[workspace] = cred.to_dict()
+        with open(auth_filename, 'w+') as file:
+            cp.write(file)
 
     @staticmethod
-    def factory(conf: dict) -> ApiAuthConf:
+    def find_user_credentials(auth_filename: str) -> str:
+        if auth_filename:
+            return auth_filename
+        elif 'BIGEYE_API_CRED_FILE' in os.environ:
+            return os.environ['BIGEYE_API_CRED_FILE']
+        elif os.path.exists(DEFAULT_CRED_FILE):
+            return DEFAULT_CRED_FILE
+        elif os.path.exists(OLD_DEFAULT_CRED_FILE):
+            return OLD_DEFAULT_CRED_FILE
+        else:
+            create_subdir_if_not_exists(path=DEFAULT_CRED_FILE, includes_file=True)
+            return DEFAULT_CRED_FILE
+
+    @classmethod
+    def find_and_parse_user_credentials(cls, auth_filename: str) -> configparser.ConfigParser:
+        cp = configparser.ConfigParser(default_section=None, interpolation=None)
+        cp.read(cls.find_user_credentials(auth_filename))
+        return cp
+
+    @staticmethod
+    def factory(conf: dict) -> ApiAuth:
         ex: Exception
 
-        for model_cls in [BasicAuthRequestLibConf, BrowserAuthConf]:
+        for model_cls in [BasicAPIAuth, BrowserAPIAuth]:
             try:
                 return model_cls(**conf)
             except Exception as e:
@@ -79,9 +140,14 @@ class ApiAuthConf(YamlSerializable):
 
 
 @dataclass
-class BasicAuthRequestLibConf(ApiAuthConf):
+class BasicAPIAuth(ApiAuth):
     """
-    Configuration models to bind required host and authentication details for data_modules runtimes.
+    Credential models to bind required host and authentication details for data_modules runtimes.
+
+    Attributes:
+        base_url: str - the full url of the bigeye instance.
+        user: str - the username to access bigeye.
+        password: str - the user password to access bigeye.
     """
     base_url: str
     """Host URL for the Bigeye -- e.g. app.bigeye.com"""
@@ -90,21 +156,27 @@ class BasicAuthRequestLibConf(ApiAuthConf):
     password: str
     """Basic Auth password for this host."""
 
+    def get_auth_headers(self) -> dict:
+        return {'auth': HTTPBasicAuth(self.user.lower(), self.password)}
+
 
 @dataclass
-class BrowserAuthConf(ApiAuthConf):
+class BrowserAPIAuth(ApiAuth):
     """
-    Configuration to bind required host and auth details from browser cookies.
+    Credential model to bind required host and auth details from browser cookies.
 
     Attributes:
         browser: BrowserType
-        browser_profile_user_name: str - the user name associated with the browser profile.
+        browser_profile_user_name: str - the username associated with the browser profile.
         base_url: str - the full url of the bigeye instance.
     """
     browser: BrowserType
     browser_profile_user_name: str = None
     base_url: str = "https://app.bigeye.com"
 
+    def get_auth_headers(self) -> dict:
+        return {'cookies': self.auth_factory().get_cookies()}
+
     def get_use_default_profile(self) -> bool:
         return False if self.browser_profile_user_name else True
 
@@ -146,7 +218,7 @@ class ChromiumBased(BrowserAuth):
     _cookie_encryption_pass: str = None
     _iterations: int = None
 
-    def __init__(self, api_conf: BrowserAuthConf):
+    def __init__(self, api_conf: BrowserAPIAuth):
         super().__init__()
         self.browser_profile_user_name = api_conf.browser_profile_user_name
         self.browser = api_conf.browser.name
@@ -314,7 +386,7 @@ class Firefox(BrowserAuth):
     osx_user_data_path: str = f'{HOME_DIR}/Library/Application Support/Firefox'
     linux_user_data_path: str = f'{HOME_DIR}/.mozilla/firefox'
 
-    def __init__(self, api_conf: BrowserAuthConf):
+    def __init__(self, api_conf: BrowserAPIAuth):
         super().__init__()
         self.domain_name = api_conf.get_domain_name()
         cookie_file = self.__find_cookie_file()
