@@ -32,3 +32,11 @@ UNKNOWN_SERIALIZATION_ERROR = "Unknown Serialization Error: value could not be s
 TWO_SCHEDULES_DEFINED = "Both schedule frequency and metric schedule are defined. Please choose one."
 MISMATCHED_ATTRIBUTE_ACROSS_MULTIPLE_FILES = \
     "Mismatched attribute {attribute}: This value must be consistent across files."
+MUST_HAVE_COLUMN_SELECTOR_NAME_OR_TYPE = "Invalid column_selector: '{column_selector}'. Column selectors must have " \
+                                         "either a name or a type specified, not just an exclusion. "
+COLUMN_SELECTOR_MUST_HAVE_VALID_REGEX = "Invalid regex for column_selector. Regular expression fails to compile with " \
+                                        "given error. {error_message}"
+NAME_AND_EXCLUDE_MUST_NOT_BE_DECLARED_IF_REGEX = "Invalid column_selector: '{column_selector}'.\nRegex selectors " \
+                                                 "cannot be provided in addition to name and exclude selectors. " \
+                                                 "Please choose either regex or name selectors."
+INVALID_THRESHOLD_BOUNDS = "Invalid threshold bounds. Both upper_bound_only and lower_bound_only are set to true."
