@@ -3,50 +3,142 @@ from __future__ import annotations
 import time
 from abc import ABC
 from json import JSONDecodeError
-from typing import List, Optional, Dict, Tuple, Union
+from typing import List, Optional, Dict, Tuple
 
 import requests
 from requests.auth import HTTPBasicAuth
 
-from bigeye_sdk.authentication.api_authentication import BasicAuthRequestLibConf, BrowserAuthConf, ApiAuthConf
+from bigeye_sdk.authentication.api_authentication import BasicAPIAuth, BrowserAPIAuth, ApiAuth
 from bigeye_sdk.bigconfig_validation.big_config_reports import MetricSuiteReport, ProcessStage
 from bigeye_sdk.client.base_client import BaseApiClient
 from bigeye_sdk.client.enum import Method
+from bigeye_sdk.authentication.config import WorkspaceConfig
 from bigeye_sdk.client.generated_datawatch_client import GeneratedDatawatchClient
 from bigeye_sdk.exceptions.exceptions import NoSourcesFoundException, TableNotFoundException, \
-    BigconfigIncompleteException
+    BigconfigIncompleteException, WorkspaceNotSetException
 from bigeye_sdk.functions.delta_functions import infer_column_mappings, build_ccm, match_tables_by_name
 from bigeye_sdk.functions.metric_functions import set_default_model_type_for_threshold, is_freshness_metric
 from bigeye_sdk.functions.table_functions import get_table_column_priority_first, table_has_metric_time, \
     fully_qualified_table_to_elements
-from bigeye_sdk.generated.com.bigeye.models.generated import MetricCreationState, \
-    MetricConfiguration, TimeInterval, Threshold, MetricType, MetricParameter, LookbackType, NotificationChannel, \
+from bigeye_sdk.generated.com.bigeye.models.generated import MetricConfiguration, MetricType, \
     CreateComparisonTableResponse, CreateComparisonTableRequest, \
     ComparisonTableConfiguration, IdAndDisplayName, ComparisonColumnMapping, \
     ColumnNamePair, Table, GetDebugQueriesResponse, Schema, Source, MetricSuite, MetricSuiteResponse, \
-    MetricNamesResponse, WorkflowStatusResponse, WorkflowProcessingStatus
+    MetricNamesResponse, WorkflowStatusResponse, WorkflowProcessingStatus, UserAuth, BigconfigWorkflowStatusResponse, \
+    MetricCreationState, LookbackType, TimeInterval, MetricParameter, NotificationChannel, Threshold
 from bigeye_sdk.log import get_logger
 from bigeye_sdk.model.delta_facade import SimpleDeltaConfiguration, SimpleDeltaConfigurationFile
 from bigeye_sdk.model.metric_facade import SimpleUpsertMetricRequest
 from bigeye_sdk.model.protobuf_extensions import MetricDebugQueries
-from bigeye_sdk.model.protobuf_message_facade import SimpleColumnMapping, SimpleColumnPair
+from bigeye_sdk.model.protobuf_message_facade import (
+    SimpleColumnMapping,
+    SimpleColumnPair,
+    SimpleNamedSchedule,
+)
 
 log = get_logger(__file__)
 
 
-def datawatch_client_factory(auth: ApiAuthConf):
-    if isinstance(auth, BasicAuthRequestLibConf):
-        client = BasicAuthDatawatchClient(api_conf=auth)
-    elif isinstance(auth, BrowserAuthConf):
-        client = BrowserAuthDatawatchClient(api_conf=auth)
+def datawatch_client_factory(auth: ApiAuth, workspace_config: WorkspaceConfig = None, workspace_id: int = None):
+    if isinstance(auth, (BasicAPIAuth, BrowserAPIAuth)):
+        if workspace_config:
+            client = DatawatchClient(auth=auth, workspace_config=workspace_config)
+        elif workspace_id:
+            client = DatawatchClient(auth=auth, workspace_config=WorkspaceConfig(workspace_id=workspace_id))
+        else:
+            accessible_workspaces = get_user_auth(auth).workspaces
+            if len(accessible_workspaces) == 1:
+                # User has access to only 1 workspace, fallback to that
+                log.info(f"No workspace provided, defaulting to only valid workspace: {accessible_workspaces[0].name}")
+                workspace_config = WorkspaceConfig(workspace_id=accessible_workspaces[0].id)
+                client = DatawatchClient(auth=auth, workspace_config=workspace_config)
+            elif len(accessible_workspaces) > 1:
+                # User has access to more than 1 workspace, need to confirm correct one
+                raise WorkspaceNotSetException(f'Expected either workspace_config or workspace_id args to be passed.')
+            else:
+                # User has access to no workspaces - throw an error to let them know
+                raise WorkspaceNotSetException(f'No workspace access detected. Please check with your Bigeye '
+                                               'administrators to obtain access to a workspace.')
     else:
         raise Exception('Auth type not supported.')
-
     return client
 
 
+def get_user_auth(auth: ApiAuth) -> UserAuth:
+    url = '/auth'
+    headers = {"Content-Type": "application/json",
+               "Accept": "application/json"}
+    if isinstance(auth, BasicAPIAuth):
+        # basic auth not supported for /auth endpoint
+        # need to get cookies by first posting login request in session
+        session = requests.Session()
+        login = {"basic-email": (None, auth.user),
+                 "basic-password": (None, auth.password)}
+        session.post(url=f'{auth.base_url}/ajaxsignin',
+                     files=login)
+        response = session.get(
+            url=f'{auth.base_url}{url}',
+            headers=headers)
+    elif isinstance(auth, BrowserAPIAuth):
+        response = requests.get(
+            url=f'{auth.base_url}{url}',
+            cookies=auth.auth_factory().get_cookies(),
+            headers=headers)
+
+    return UserAuth().from_dict(response.json())
+
+
 class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
 
+    def __init__(self, auth: BasicAPIAuth | BrowserAPIAuth, workspace_config: WorkspaceConfig):
+        super().__init__(base_url=auth.base_url)
+        self._auth = auth
+        self.config = workspace_config
+
+    def _call_datawatch_impl(self, method: Method, url, body: str = None, params: dict = None):
+        try:
+            fq_url = f'{self._base_url}{url}'
+            log.info(f'Request Type: {method.name}; URL: {fq_url}; Body: {body}')
+            headers = {"Content-Type": "application/json",
+                       "Accept": "application/json",
+                       "x-bigeye-workspace-id": f'{self.config.workspace_id}'}
+
+            kwargs = {
+                'url': fq_url,
+                'headers': headers,
+                'data': body,
+                'params': params
+            }
+
+            kwargs.update(self._auth.get_auth_headers())
+
+            if method == Method.GET:
+                response = requests.get(**kwargs)
+            elif method == Method.POST:
+                response = requests.post(**kwargs)
+            elif method == Method.PUT:
+                response = requests.put(**kwargs)
+            elif method == Method.DELETE:
+                response = requests.delete(**kwargs)
+            else:
+                raise Exception(f'Unsupported http method {method}')
+        except Exception as e:
+            log.error(f'Exception calling datawatch: {str(e)}')
+            raise e
+        else:
+            log.info(f'Return Code: {response.status_code}')
+            if response.status_code < 200 or response.status_code >= 300:
+                log.error(f'Error code returned from datawatch: {response.status_code} - {response.reason}')
+                raise Exception(response.text)
+            else:
+                # Not empty response
+                if response.status_code != 204:
+                    try:
+                        return response.json()
+                    except JSONDecodeError as e:
+                        log.info(f'Cannot decode response.  {response}')
+                        return ''
+
     def delete_metrics(self, metrics: List[MetricConfiguration] = None):
         """
         Deletes multiple metrics.
@@ -302,7 +394,7 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
             cron_resp = self.get_named_schedule(ids=[r.named_schedule.id])
 
             if cron_resp.named_schedules:
-                response_as_sdc.cron_schedule = cron_resp.named_schedules[0]
+                response_as_sdc.cron_schedule = SimpleNamedSchedule.from_datawatch_object(cron_resp.named_schedules[0])
 
             created_deltas.append(response_as_sdc)
 
@@ -335,16 +427,17 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
         self.delete_deltas_by_name(delta_names=[sdc.delta_name for sdc in sdcl])
 
         responses = []
-        table_names = []
-        schemas = []
-        table_ids = []
-        source_schema: str
-        target_schema: str
-        source_tbl: str = ""
-        target_tbl: str = ""
-        source_table = Table()
-        target_table = Table()
+
         for sdc in sdcl:
+            table_names = []
+            schemas = []
+            table_ids = []
+            source_schema: str
+            target_schema: str
+            source_tbl: str = ""
+            target_tbl: str = ""
+            source_table = Table()
+            target_table = Table()
 
             if sdc.fq_source_table_name:
                 swh, source_schema, source_tbl = fully_qualified_table_to_elements(sdc.fq_source_table_name)
@@ -408,15 +501,12 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
             response_as_ctc.target_filters = sdc.target_filters
 
             schedules = self.get_named_schedule().named_schedules
-
             for s in schedules:
                 """Matches on cron value."""
-                if s.cron == sdc.cron_schedule.cron or s.name == sdc.cron_schedule.name:
+                if sdc.cron_schedule is None:
+                    response_as_ctc.named_schedule = IdAndDisplayName(id=s.id, display_name=s.name)
+                elif s.cron == sdc.cron_schedule.cron or s.name == sdc.cron_schedule.name:
                     response_as_ctc.named_schedule = IdAndDisplayName(id=s.id, display_name=s.name)
-
-            if response_as_ctc.named_schedule.id == 0:
-                ns = self.create_named_schedule(name=sdc.cron_schedule.name, cron=sdc.cron_schedule.cron)
-                response_as_ctc.named_schedule = IdAndDisplayName(id=ns.id, display_name=ns.name)
 
             response = self.create_delta(comparison_table_configuration=response_as_ctc)
 
@@ -554,51 +644,33 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
             self._call_datawatch(Method.POST, url=url, body=metric_suite.to_json())
         )
 
-    def __get_metric_suite_response_for_queue(self, metric_suite: MetricSuite, mc_url: str) -> MetricSuiteResponse:
-
-        suite_response: MetricSuiteResponse = MetricSuiteResponse()
-        # TODO: Update once backend returns expected response from queue
-        """Silently plan to obtain the MetricSuiteResponse, in order to generate reports for users."""
-        if isinstance(self, BasicAuthDatawatchClient):
-            suite_response.from_dict(requests.post(
-                f"{self.get_base_url()}{mc_url}/dry-run",
-                headers={"Content-Type": "application/json", "Accept": "application/json"},
-                data=metric_suite.to_json(),
-                auth=self._auth
-            ).json())
-        elif isinstance(self, BrowserAuthDatawatchClient):
-            suite_response.from_dict(requests.post(
-                f"{self.get_base_url()}{mc_url}/dry-run",
-                headers={"Content-Type": "application/json", "Accept": "application/json"},
-                data=metric_suite.to_json(),
-                cookies=self._cookies
-            ).json())
-
-        return suite_response
-
-    def post_metric_suite_queue(self, metric_suite: MetricSuite) -> MetricSuiteResponse:
-        mc_url = "/api/v1/metric-suites"
-
-        suite_response = self.__get_metric_suite_response_for_queue(metric_suite=metric_suite, mc_url=mc_url)
+    def post_metric_suite_queue(self, metric_suite: MetricSuite, apply: bool = False) -> MetricSuiteResponse:
+        if apply:
+            url = "/api/v1/metric-suites/queue"
+        else:
+            url = f"/api/v1/metric-suites/dry-run/queue"
 
+        # TODO update once backend returns a BigconfigWorkflowStatusResponse for inital call to queue
         response: WorkflowStatusResponse = WorkflowStatusResponse().from_dict(
-            self._call_datawatch(Method.POST, url=f"{mc_url}/queue", body=metric_suite.to_json())
+            self._call_datawatch(Method.POST, url=url, body=metric_suite.to_json())
         )
 
-        status_url = f"/api/v1/workflows/{response.workflow_id}/status"
+        status_url = f"/api/v1/metric-suites/status/{response.workflow_id}"
         while response.status == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_QUEUED or \
                 response.status == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_IN_PROGRESS:
             log.info(f"Bigconfig in queue...")
             time.sleep(10)
-            wsr = WorkflowStatusResponse()
-            r: WorkflowStatusResponse = wsr.from_dict(self._call_datawatch(Method.GET, url=status_url))
-            response.status = r.status
+            bwsr: BigconfigWorkflowStatusResponse = BigconfigWorkflowStatusResponse().from_dict(
+                self._call_datawatch(Method.GET, url=status_url))
+            response.status = bwsr.workflow_status_response.status
+
         log.info(f"Queuing complete. Workflow finished with status {response.status.name}")
+
         if response.status != WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_COMPLETED:
             err = f"Bigconfig workflow was not completed. Final status: {response.status.name}"
             raise BigconfigIncompleteException(err)
 
-        return suite_response
+        return bwsr.metric_suite_response
 
     def upsert_metric(
             self,
@@ -751,114 +823,3 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
 
         return r
 
-
-class BasicAuthDatawatchClient(DatawatchClient):
-
-    def __init__(self, api_conf: BasicAuthRequestLibConf):
-        super().__init__(base_url=api_conf.base_url)
-        self._auth = HTTPBasicAuth(api_conf.user, api_conf.password)
-        pass
-
-    def _call_datawatch_impl(self, method: Method, url, body: str = None, params: dict = None):
-        try:
-            fq_url = f'{self._base_url}{url}'
-            log.info(f'Request Type: {method.name}; URL: {fq_url}; Body: {body}')
-            if method == Method.GET:
-                response = requests.get(
-                    fq_url,
-                    auth=self._auth,
-                    params=params
-                )
-            elif method == Method.POST:
-                response = requests.post(
-                    fq_url,
-                    headers={"Content-Type": "application/json", "Accept": "application/json"},
-                    data=body,
-                    auth=self._auth,
-                    params=params
-                )
-            elif method == Method.PUT:
-                response = requests.put(
-                    fq_url,
-                    headers={"Content-Type": "application/json", "Accept": "application/json"},
-                    data=body,
-                    auth=self._auth,
-                    params=params
-                )
-            elif method == Method.DELETE:
-                response = requests.delete(f'{self._base_url}{url}',
-                                           headers={"Content-Type": "application/json", "Accept": "application/json"},
-                                           data=body,
-                                           auth=self._auth, params=params)
-            else:
-                raise Exception(f'Unsupported http method {method}')
-        except Exception as e:
-            log.info(f'URL: {response.url}')
-            log.error(f'Exception calling datawatch: {str(e)}')
-            raise e
-        else:
-            log.info(f'URL: {response.url}')
-            log.info(f'Return Code: {response.status_code}')
-            if response.status_code < 200 or response.status_code >= 300:
-                errmsg = f'Error Code {response.status_code}: {response.text}'
-                log.error(errmsg)
-                raise Exception(errmsg)
-            else:
-                # Not empty response
-                try:
-                    if response.status_code != 204:
-                        return response.json()
-                except JSONDecodeError as e:
-                    log.info(f'Cannot decode response.  {response}')
-                    return ''
-
-
-class BrowserAuthDatawatchClient(DatawatchClient):
-    def __init__(self, api_conf: BrowserAuthConf):
-        super().__init__(base_url=api_conf.base_url)
-        self.api_conf = api_conf
-        self._cookies = self.api_conf.auth_factory().get_cookies()
-
-    def _call_datawatch_impl(self, method: Method, url, body: str = None, params: dict = None):
-        # TODO: This shouldn't repeat the above code when the only thing changing is a call to cookies, instead of auth.
-        try:
-            fq_url = f'{self._base_url}{url}'
-            log.info(f'Request Type: {method.name}; URL: {fq_url}; Body: {body}')
-            if method == Method.GET:
-                response = requests.get(
-                    fq_url,
-                    cookies=self._cookies
-                )
-            elif method == Method.POST:
-                response = requests.post(
-                    fq_url,
-                    headers={"Content-Type": "application/json", "Accept": "application/json"},
-                    data=body,
-                    cookies=self._cookies
-                )
-            elif method == Method.PUT:
-                response = requests.put(
-                    fq_url,
-                    headers={"Content-Type": "application/json", "Accept": "application/json"},
-                    data=body,
-                    cookies=self._cookies
-                )
-            elif method == Method.DELETE:
-                response = requests.delete(f'{self._base_url}{url}',
-                                           headers={"Content-Type": "application/json", "Accept": "application/json"},
-                                           data=body,
-                                           cookies=self._cookies)
-            else:
-                raise Exception(f'Unsupported http method {method}')
-        except Exception as e:
-            log.error(f'Exception calling datawatch: {str(e)}')
-            raise e
-        else:
-            log.info(f'Return Code: {response.status_code}')
-            if response.status_code < 200 or response.status_code >= 300:
-                log.error(f'Error code returned from datawatch: {response.status_code} - {response.reason}')
-                raise Exception(response.text)
-            else:
-                # Not empty response
-                if response.status_code != 204:
-                    return response.json()
