@@ -1,25 +1,31 @@
 import os.path
 import sys
 from pathlib import Path
-from typing import List, Dict, Tuple, Union
+from typing import List, Dict, Tuple, Union, Optional
+
+from rich.prompt import Confirm
+from rich.text import Text
 
 from bigeye_sdk.bigconfig_validation.big_config_reports import raise_files_contain_error_exception, MetricSuiteReport, \
-    process_reports, ProcessStage
+    process_reports, ProcessStage, REPORTS
 from bigeye_sdk.bigconfig_validation.validation_context import process_validation_errors, \
     get_validation_error_cnt, get_all_validation_errors_flat
 from bigeye_sdk.bigconfig_validation.yaml_validation_error_messages import SRC_NOT_EXISTS_FOR_DEPLOYMENT_ERRMSG, \
     METRIC_APPLICATION_ERROR, MISMATCHED_ATTRIBUTE_ACROSS_MULTIPLE_FILES
 from bigeye_sdk.client.datawatch_client import DatawatchClient
-from bigeye_sdk.exceptions.exceptions import FileLoadException, BigConfigValidationException, NoSourcesFoundException
+from bigeye_sdk.exceptions.exceptions import FileLoadException, BigConfigValidationException, NoSourcesFoundException, \
+    FileNotFoundException
 from bigeye_sdk.functions.metric_functions import _is_table_level_metric
 from bigeye_sdk.functions.search_and_match_functions import wildcard_search
 from bigeye_sdk.generated.com.bigeye.models.generated import Source, CohortDefinition, \
-    MetricSuite, CohortAndMetricDefinition, CatalogEntityType, MetricDefinition, FieldType, NamedSchedule
+    MetricSuite, CohortAndMetricDefinition, CatalogEntityType, MetricDefinition, FieldType, NamedSchedule, User
 from bigeye_sdk.log import get_logger
 from bigeye_sdk.model.big_config import BigConfig, RowCreationTimes, TagDeployment, TableDeployment, \
-    SavedMetricDefinitions
+    SavedMetricDefinitions, ColumnSelector
 from bigeye_sdk.model.protobuf_enum_facade import SimplePredefinedMetricName, SimpleLookbackType
-from bigeye_sdk.model.protobuf_message_facade import SimpleCollection, SimpleMetricDefinition, SimpleSLA, SimpleLookback
+from bigeye_sdk.model.protobuf_message_facade import SimpleCollection, SimpleMetricDefinition, SimpleSLA, \
+    SimpleLookback
+
 from bigeye_sdk.serializable import File, BIGCONFIG_FILE
 
 log = get_logger(__file__)
@@ -44,39 +50,47 @@ def get_fq_name_from_cohort(cohort: CohortDefinition, source_name: str = None):
         return f'{source_name}.{r}'
 
 
-def _find_bigconfig_files(source_paths: List[str] = [], recursive: bool = False) -> List[BIGCONFIG_FILE]:
+def _find_bigconfig_files(input_paths: List[str] = [], recursive: bool = False) -> List[BIGCONFIG_FILE]:
     """
-    Finds bigconfig files either in specified source path or in working directory
+    Finds bigconfig files either in specified input path or in working directory
     Args:
-        source_paths: specify List of source paths or working directory will be used.
+        input_paths: specify List of input paths or working directory will be used.
 
     Returns: None
 
     """
     files: List[Path] = []
-    for sp in source_paths:
-        if os.path.isfile(sp):
-            files.append(Path(sp))
+    for ip in input_paths:
+        if os.path.isfile(ip):
+            files.append(Path(ip))
         elif recursive:
-            files.extend(list(Path(sp).rglob('*.y*ml')))
+            files.extend(list(Path(ip).rglob('*.y*ml')))
         else:
-            files.extend(list(Path(sp).glob('*.y*ml')))
+            files.extend(list(Path(ip).glob('*.y*ml')))
 
     if not files:
-        """Assumes driver run in directory containing Bigconfig YAML."""
-        files = list(Path.cwd().rglob('*.y*ml')) if recursive else list(Path.cwd().glob('*.y*ml'))
+        """No YAML files were found at the given input path or current working directory, raise error for user to fix input paths."""
+        raise FileNotFoundException(f'No YAML files for the given input paths or current working directory were found.')
 
     bigeye_files: List[BIGCONFIG_FILE] = []
     file: BIGCONFIG_FILE
+    load_errs = []
     for file in files:
         try:
-            """Loading Bigconfig YAML.  If file is not of Bigconfig type then the error will be caught and passed."""
+            """
+            Loading Bigconfig YAML.  If file is not of Bigconfig type then the error will be caught and passed. If file
+            is not valid YAML, then an exception will be thrown.
+            """
             bigeye_files.append(File.load(str(file)))
         except FileLoadException as e:
-            log.warning(f'File {file} failed to load with error: {str(e.message)}')
+            load_err = f'File {file} failed to load with error: {str(e.message)}'
+            load_errs.append(load_err)
+            log.warning(load_err)
 
     if len(bigeye_files) == 0:
-        raise BigConfigValidationException(f'No conforming files found. There may be a formatting issue with the yaml.')
+        errors = "\n\n=========\n\n".join(load_errs)
+        err = f'No conforming files found. There may be a formatting issue with the yaml.\nErrors found:\n{errors}'
+        raise BigConfigValidationException(err)
 
     else:
         return bigeye_files
@@ -89,6 +103,7 @@ class MetricSuiteController:
 
         self.sources_by_name_ix: Dict[str, Source] = self.client.get_sources_by_name()
         self.sources_by_id_ix: Dict[int, Source] = {v.id: v for k, v in self.sources_by_name_ix.items()}
+        self.bigeye_users: Optional[Dict[str, int]] = None
 
         self.table_level_metrics: List[SimplePredefinedMetricName] = [
             SimplePredefinedMetricName.from_datawatch_object(pmn)
@@ -109,45 +124,58 @@ class MetricSuiteController:
 
         """
         existing_collections = {c.name: c for c in self.client.get_collections().collections}
+        merged_or_new: SimpleCollection
         deployment_collections = []
-        if apply:
-            for collection in bigconfig.get_collections():
-                if collection.name in existing_collections.keys():
-                    existing = SimpleCollection.from_datawatch_object(existing_collections[collection.name])
-                    merged = collection.merge_for_upsert(existing=existing, overwrite=overwrite)
-                    deployment_collections.append(merged)
-                    self.client.update_collection(collection=merged.to_datawatch_object())
-                else:
-                    c = collection.to_datawatch_object()
-                    c = self.client.create_collection(
-                        collection_name=c.name,
-                        description=c.description,
-                        metric_ids=c.metric_ids,
-                        notification_channels=c.notification_channels,
-                        muted_until_timestamp=c.muted_until_timestamp
-                    ).collection
-                    deployment_collections.append(SimpleCollection.from_datawatch_object(c))
+
+        for collection in bigconfig.get_collections():
+            if collection.name in existing_collections.keys():
+                """If the collection exists, send the collection ID as part of a plan and only update for apply"""
+                existing = SimpleCollection.from_datawatch_object(existing_collections[collection.name])
+                merged_or_new = collection.merge_for_upsert(existing=existing)
+                deployment_collections.append(merged_or_new)
+                if apply:
+                    self.client.update_collection(collection=merged_or_new.to_datawatch_object())
+            elif apply:
+                """If it doesn't exist, only create it during apply"""
+                c = collection.to_datawatch_object()
+                c = self.client.create_collection(
+                    collection_name=c.name,
+                    description=c.description,
+                    metric_ids=c.metric_ids,
+                    notification_channels=c.notification_channels,
+                    muted_until_timestamp=c.muted_until_timestamp
+                ).collection
+                merged_or_new = SimpleCollection.from_datawatch_object(c)
+                deployment_collections.append(merged_or_new)
 
         return deployment_collections
 
+    def _get_matching_source_ids(self, column_selector: ColumnSelector, source_include_pattern: str) -> List[int]:
+        source_ids_to_include = [source.id
+                                 for source_name, source in self.sources_by_name_ix.items()
+                                 if source_name in wildcard_search(search_string=source_include_pattern,
+                                                                   content=[source_name],
+                                                                   is_raw_regex=column_selector.is_raw_regex())
+                                 ]
+        return source_ids_to_include
+
     def row_creation_times_to_cohort(self, row_creation_times: RowCreationTimes) -> Dict[int, List[CohortDefinition]]:
         r: Dict[int, List[CohortDefinition]] = {}
 
         for cs in row_creation_times.column_selectors:
-            source_pattern, schema_pattern, table_pattern, column_pattern = cs.explode_name_to_cohort_patterns()
+            selector = cs.regex if cs.is_raw_regex() else cs.name
+            source_pattern, schema_pattern, table_pattern, column_pattern = cs.explode_to_cohort_patterns(selector)
+            source_exclusions, schema_exclusions, table_exclusions, column_exclusions = cs.explode_exclusions_to_cohort_patterns()
 
-            matching_source_ids = [source.id
-                                   for source_name, source in self.sources_by_name_ix.items()
-                                   if source_name in wildcard_search(search_string=source_pattern,
-                                                                     content=[source_name])
-                                   ]
+            matching_source_ids = self._get_matching_source_ids(column_selector=cs,
+                                                                source_include_pattern=source_pattern)
 
             if not matching_source_ids:
                 "registering validation errors when the source was not matched."
-                errlns = cs.get_error_lines()
+                error_lines = cs.get_error_lines()
                 row_creation_times.register_validation_error(
-                    error_lines=errlns,
-                    error_message=SRC_NOT_EXISTS_FOR_DEPLOYMENT_ERRMSG.format(fq_name=cs))
+                    error_lines=error_lines,
+                    error_message=SRC_NOT_EXISTS_FOR_DEPLOYMENT_ERRMSG.format(fq_name=source_pattern))
 
             for id in matching_source_ids:
                 cd = CohortDefinition(schema_name_pattern=schema_pattern,
@@ -155,7 +183,11 @@ class MetricSuiteController:
                                       column_name_pattern=column_pattern,
                                       column_type=cs.type.to_datawatch_object() if cs.type
                                       else FieldType.FIELD_TYPE_UNSPECIFIED,
-                                      entity_type=CatalogEntityType.CATALOG_ENTITY_TYPE_FIELD)
+                                      schema_exclusions=schema_exclusions,
+                                      table_exclusions=table_exclusions,
+                                      column_exclusions=column_exclusions,
+                                      entity_type=CatalogEntityType.CATALOG_ENTITY_TYPE_FIELD,
+                                      use_raw_regex=cs.is_raw_regex())
                 if id in r.keys():
                     r[id].append(cd)
                 else:
@@ -195,6 +227,29 @@ class MetricSuiteController:
 
         return r
 
+    def _generate_tags_for_favorites(self, user_favorites: List[str]) -> Optional[List[str]]:
+        """
+        Given a list of user emails, this will build a list of strings that include the corresponding Bigeye user id.
+        If no user id is found for a given email, then a warning message will be provided.
+        """
+        if not user_favorites:
+            return None
+
+        if not self.bigeye_users:
+            # only make this request once and only if a user adds favorites
+            self.bigeye_users = {u.email: u.id for u in self.client.get_users().users}
+
+        favorite_tags: List[str] = []
+        for uf in user_favorites:
+            matching_user_id = self.bigeye_users.get(uf, None)
+            if not matching_user_id:
+                log.warning(f'No user with email `{uf}` could be found. Skipping metric favorites for user. Please '
+                            f'check the spelling of the email provided.')
+            else:
+                favorite_tags.append(f'favorite_user-{matching_user_id}')
+
+        return favorite_tags
+
     def tag_deployment_to_cohort_and_metric_def(self,
                                                 tag_deployment: TagDeployment,
                                                 named_schedules: List[NamedSchedule],
@@ -225,6 +280,8 @@ class MetricSuiteController:
             if m.metric_schedule and m.metric_schedule.named_schedule:
                 m.metric_schedule.named_schedule.id = [s.id for s in named_schedules
                                                        if s.name == m.metric_schedule.named_schedule.name][0]
+            m.user_favorites = self._generate_tags_for_favorites(m.user_favorites)
+
             if not m.lookback:
                 m.lookback = default_lookback
 
@@ -236,13 +293,13 @@ class MetricSuiteController:
                 column_metrics.append((m, mdwo))
 
         for cs in tag_deployment.column_selectors:
-            source_pattern, schema_pattern, table_pattern, column_pattern = cs.explode_name_to_cohort_patterns()
+            selector = cs.regex if cs.is_raw_regex() else cs.name
+            source_pattern, schema_pattern, table_pattern, column_pattern = cs.explode_to_cohort_patterns(selector)
+            source_exclusions, schema_exclusions, table_exclusions, column_exclusions = cs.explode_exclusions_to_cohort_patterns()
+
+            matching_source_ids = self._get_matching_source_ids(column_selector=cs,
+                                                                source_include_pattern=source_pattern)
 
-            matching_source_ids = [source.id
-                                   for source_name, source in self.sources_by_name_ix.items()
-                                   if source_name in wildcard_search(search_string=source_pattern,
-                                                                     content=[source_name])
-                                   ]
             if not matching_source_ids:
                 "registering validation errors when the source was not matched."
                 tag_deployment.register_validation_error(
@@ -256,7 +313,11 @@ class MetricSuiteController:
                                                   column_name_pattern=column_pattern,
                                                   column_type=cs.type.to_datawatch_object() if cs.type
                                                   else FieldType.FIELD_TYPE_UNSPECIFIED,
-                                                  entity_type=CatalogEntityType.CATALOG_ENTITY_TYPE_FIELD)
+                                                  schema_exclusions=schema_exclusions,
+                                                  table_exclusions=table_exclusions,
+                                                  column_exclusions=column_exclusions,
+                                                  entity_type=CatalogEntityType.CATALOG_ENTITY_TYPE_FIELD,
+                                                  use_raw_regex=cs.is_raw_regex())
                 for sid in matching_source_ids:
                     """Tag deployments support source patterns.  Each source id becomes a key in the returned dictionary 
                     with the exact same cohorts and metrics."""
@@ -267,21 +328,26 @@ class MetricSuiteController:
                         cmds[sid] = [cmd]
 
             if table_metrics:
-                if column_pattern != '*':
+                if (column_pattern != '*' and cs.name) or (column_pattern != '.*' and cs.regex):
                     for smd, md in table_metrics:
-                        errmsg = f"Table level metrics can only be applied to column selectors if the column is a " \
-                                 f"wild card.  Column Selector: {cs.name}.  Metric: {smd.metric_type}.  " \
+                        errmsg = f"Table level metrics can only be applied to column selectors if the column pattern is " \
+                                 f"only a wild card.  Column Pattern: {column_pattern} -- Metric: {smd.metric_type}.  " \
                                  f"Table level metrics include: {', '.join([i.name for i in self.table_level_metrics])}"
                         tag_deployment.register_validation_error(
                             error_lines=smd.get_error_lines(),
                             error_context_lines=tag_deployment.get_error_lines(),
                             error_message=METRIC_APPLICATION_ERROR.format(errmsg=errmsg))
 
-                table_cohort = CohortDefinition(schema_name_pattern=schema_pattern, table_name_pattern=table_pattern,
+                table_cohort = CohortDefinition(schema_name_pattern=schema_pattern,
+                                                table_name_pattern=table_pattern,
                                                 column_name_pattern=column_pattern,
                                                 column_type=cs.type.to_datawatch_object() if cs.type
                                                 else FieldType.FIELD_TYPE_UNSPECIFIED,
-                                                entity_type=CatalogEntityType.CATALOG_ENTITY_TYPE_DATASET)
+                                                schema_exclusions=schema_exclusions,
+                                                table_exclusions=table_exclusions,
+                                                column_exclusions=column_exclusions,
+                                                entity_type=CatalogEntityType.CATALOG_ENTITY_TYPE_DATASET,
+                                                use_raw_regex=cs.is_raw_regex())
 
                 for sid in matching_source_ids:
                     """Tag deployments support source patterns.  Each source id becomes a key in the returned dictionary 
@@ -298,7 +364,7 @@ class MetricSuiteController:
                                                   table_deployment: TableDeployment,
                                                   named_schedules: List[NamedSchedule],
                                                   default_lookback: SimpleLookback,
-                                                  deployment_sla: Union[SimpleSLA, SimpleCollection] = None) -> \
+                                                  deployment_collection: Union[SimpleSLA, SimpleCollection] = None) -> \
             Dict[int, List[CohortAndMetricDefinition]]:
         """
         Builds a Cohort and MetricDefinition from a TableDeployment object
@@ -306,7 +372,7 @@ class MetricSuiteController:
             table_deployment: table deployment from which to generate cohort and metric definition
             named_schedules: a list of named schedules from the workspace
             default_lookback: the lookback corresponding to "metric.data_time_window.default"
-            deployment_sla: SLA to which metrics will be added.
+            deployment_collection: Collection to which metrics will be added.
 
         Returns: Dict[warehouse_id: int, CohortAndMetricDefinition]
         """
@@ -347,13 +413,17 @@ class MetricSuiteController:
                     error_message=METRIC_APPLICATION_ERROR.format(errmsg=errmsg)
                 )
             else:
-                if deployment_sla and deployment_sla.id not in m.sla_ids:
-                    m.sla_ids.append(deployment_sla.id)
+                if deployment_collection and deployment_collection.id not in m.collection_ids:
+                    m.collection_ids.append(deployment_collection.id)
                 if m.metric_schedule and m.metric_schedule.named_schedule:
                     m.metric_schedule.named_schedule.id = [s.id for s in named_schedules
                                                            if s.name == m.metric_schedule.named_schedule.name][0]
+
                 if not m.lookback:
                     m.lookback = default_lookback
+
+                m.user_favorites = self._generate_tags_for_favorites(m.user_favorites)
+
                 mdwo = m.to_datawatch_object()
                 mdwo.is_table_metric = True
                 table_metrics.append(mdwo)
@@ -384,13 +454,17 @@ class MetricSuiteController:
                         error_message=METRIC_APPLICATION_ERROR.format(errmsg=errmsg)
                     )
                 else:
-                    if deployment_sla and deployment_sla.id not in m.collection_ids:
-                        m.collection_ids.append(deployment_sla.id)
+                    if deployment_collection and deployment_collection.id not in m.collection_ids:
+                        m.collection_ids.append(deployment_collection.id)
                     if m.metric_schedule and m.metric_schedule.named_schedule:
                         m.metric_schedule.named_schedule.id = [s.id for s in named_schedules
                                                                if s.name == m.metric_schedule.named_schedule.name][0]
+
                     if not m.lookback:
                         m.lookback = default_lookback
+
+                    m.user_favorites = self._generate_tags_for_favorites(m.user_favorites)
+
                     mdwo = m.to_datawatch_object()
                     col_metrics.append(mdwo)
 
@@ -425,7 +499,6 @@ class MetricSuiteController:
         named_schedules: List[NamedSchedule] = self.client.get_named_schedule().named_schedules
         default_lookback_type: bool = [ac.boolean_value for ac in self.client.get_advanced_configs()
                                        if ac.key == "metric.data_time_window.default"][0]
-
         default_lookback = SimpleLookback() if default_lookback_type \
             else SimpleLookback(lookback_type=SimpleLookbackType.DATA_TIME)
 
@@ -460,7 +533,7 @@ class MetricSuiteController:
                     table_deployment=table_d,
                     named_schedules=named_schedules,
                     default_lookback=default_lookback,
-                    deployment_sla=deployment_sla
+                    deployment_collection=deployment_sla
                 )
                 for sid, definitions in r.items():
                     """consolidate based on sources"""
@@ -513,7 +586,8 @@ class MetricSuiteController:
                                             purge_all_sources=purge_all_sources,
                                             apply=apply
                                             )
-            process_reports(output_path=output_path)
+            process_stage = ProcessStage.APPLY if apply else ProcessStage.PLAN
+            process_reports(output_path=output_path, strict_mode=False, process_stage=process_stage)
         except NoSourcesFoundException as e:
             sys.exit(e.message)
 
@@ -521,7 +595,10 @@ class MetricSuiteController:
                           input_path: List[str] = (Path.cwd()),
                           output_path: str = Path.cwd(),
                           apply: bool = False,
-                          no_queue: bool = False, recursive: bool = False):
+                          no_queue: bool = False,
+                          recursive: bool = False,
+                          strict_mode: bool = False,
+                          auto_approve: bool = False):
         """
         Executes an Apply or Plan for a Big Config.
         Args:
@@ -530,10 +607,11 @@ class MetricSuiteController:
             apply: If true then Big Config will be applied to the workspace.  If false then a plan will be generated.
             no_queue: If true, the bigconfig will be run without queuing
             recursive: If true, search for files recursively
+            strict_mode: If true errors from the API raise an exception.
         Returns: None
 
         """
-        files: List[BIGCONFIG_FILE] = _find_bigconfig_files(input_path, recursive=recursive)
+        files: List[BIGCONFIG_FILE] = _find_bigconfig_files(input_paths=input_path, recursive=recursive)
 
         bigconfig: BigConfig = combine_bigconfigs(files)
 
@@ -561,22 +639,45 @@ class MetricSuiteController:
                                                 fixme_file_list=fixme_file_list)
 
         # Applies changes and overwrites once local validations have passed.
+        if apply and not auto_approve:
+            # run a plan, ask for approval, then apply
+            self._process_metric_suites(metric_suites=metric_suites, output_path=output_path,
+                                        strict_mode=strict_mode, apply=False, no_queue=no_queue)
+            approved = Confirm.ask(Text(f'Please review expected results. Are you sure you want to continue?',
+                                        style='bold red'))
+            if approved:
+                self._process_metric_suites(metric_suites=metric_suites, output_path=output_path,
+                                            strict_mode=strict_mode, apply=apply, no_queue=no_queue)
+        else:
+            self._process_metric_suites(metric_suites=metric_suites, output_path=output_path,
+                                        strict_mode=strict_mode, apply=apply, no_queue=no_queue)
+
+    def _process_metric_suites(self,
+                               metric_suites: List[MetricSuite],
+                               output_path: str = Path.cwd(),
+                               strict_mode: bool = False,
+                               apply: bool = False,
+                               no_queue: bool = False):
+        """Send metric suites to correct endpoints and process reports."""
+
+        # if applying, clear out any plan reports that were needed for approvals.
         if apply:
-            self._upsert_collections(bigconfig=bigconfig, overwrite=True,
-                                     apply=apply)  # Will always overwrite for Bigconfig.
+            REPORTS.clear()
+
+        process_stage = ProcessStage.APPLY if apply else ProcessStage.PLAN
 
         for metric_suite in metric_suites:
-            j = metric_suite.to_json()
-            if not apply or no_queue:
+            if no_queue:
                 response = self.client.post_metric_suite(metric_suite=metric_suite, apply=apply)
             else:
-                response = self.client.post_metric_suite_queue(metric_suite=metric_suite)
-            process_stage = ProcessStage.APPLY if apply else ProcessStage.PLAN
+                response = self.client.post_metric_suite_queue(metric_suite=metric_suite, apply=apply)
+
             MetricSuiteReport.from_datawatch_object(response,
                                                     source_name=self.sources_by_id_ix[metric_suite.source_id].name,
                                                     process_stage=process_stage)
 
-        process_reports(output_path=output_path)
+        # process reports for the current stage - even if different from user issued command
+        process_reports(output_path=output_path, strict_mode=strict_mode, process_stage=process_stage)
 
 
 def combine_bigconfigs(bigconfigs: List[BigConfig]) -> BigConfig:
