@@ -3,6 +3,11 @@ class FileLoadException(Exception):
         self.message = message
 
 
+class FileNotFoundException(Exception):
+    def __init__(self, message: str):
+        self.message = message
+
+
 class AttributeTypeException(Exception):
     def __init__(self, message: str):
         self.message = message
@@ -13,6 +18,16 @@ class InvalidConfigurationException(Exception):
         self.message = message
 
 
+class ConfigNotFoundException(Exception):
+    def __init__(self, message: str):
+        self.message = message
+
+
+class ConfigParseException(Exception):
+    def __init__(self, message: str):
+        self.message = message
+
+
 class BrowserAuthException(Exception):
     def __init__(self, message: str):
         self.message = message
@@ -36,3 +51,8 @@ class TableNotFoundException(Exception):
 class BigconfigIncompleteException(Exception):
     def __init__(self, message: str):
         self.message = message
+
+
+class WorkspaceNotSetException(Exception):
+    def __init__(self, message: str):
+        self.message = message
\ No newline at end of file
