@@ -55,7 +55,7 @@ class WriteDataResult:
 
 def write_to_file(url: str, serialized_data: List[str]) -> str:
     log.info(f'Persisting file: {url}')
-    with smart_open.smart_open(url, 'wb') as fout:
+    with smart_open.open(url, 'wb') as fout:
         for s in serialized_data:
             fout.write(s.encode())
 
