@@ -14,7 +14,8 @@ from bigeye_sdk.bigconfig_validation.validation_functions import safe_split_dict
 from bigeye_sdk.bigconfig_validation.yaml_model_base import YamlModelWithValidatorContext
 from bigeye_sdk.bigconfig_validation.yaml_validation_error_messages import FORMATTING_ERRMSG, \
     OVERRIDE_METRIC_TYPE_SAVED_METRIC_ERRMSG, MUST_HAVE_METRIC_TYPE_ERRMSG, MUST_HAVE_METRIC_ID_ERRMSG, \
-    METRIC_TYPE_NOT_EXISTS_ERRMSG, POSSIBLE_MATCH_ERRMSG, NO_POSSIBLE_MATCH_ERRMSG, TWO_SCHEDULES_DEFINED
+    METRIC_TYPE_NOT_EXISTS_ERRMSG, POSSIBLE_MATCH_ERRMSG, NO_POSSIBLE_MATCH_ERRMSG, TWO_SCHEDULES_DEFINED, \
+    INVALID_THRESHOLD_BOUNDS
 from bigeye_sdk.exceptions import InvalidConfigurationException
 from bigeye_sdk.functions.search_and_match_functions import fuzzy_match
 from bigeye_sdk.functions.table_functions import get_table_column_id
@@ -22,7 +23,7 @@ from bigeye_sdk.generated.com.bigeye.models.generated import Threshold, SimpleBo
     SimpleBound, NamedSchedule, ComparisonColumnMapping, Table, IdAndDisplayName, ColumnNamePair, TimeInterval, \
     MetricParameter, MetricType, PredefinedMetric, TemplateMetric, AutoThreshold, ForecastModelType, RelativeThreshold, \
     StandardDeviationThreshold, FreshnessScheduleThreshold, NotificationChannel, MetricDefinition, MetricConfiguration, \
-    Collection, SlackChannelInfo, Webhook, WebhookHeader, MetricSchedule, TimeIntervalType
+    Collection, SlackChannelInfo, Webhook, WebhookHeader, MetricSchedule, NoneThreshold, LookbackType
 from bigeye_sdk.log import get_logger
 from bigeye_sdk.model.base_datawatch_facade import DatawatchFacade
 from bigeye_sdk.model.protobuf_enum_facade import SimplePredefinedMetricName, SimpleTimeIntervalType, \
@@ -31,6 +32,98 @@ from bigeye_sdk.serializable import PydanticSubtypeSerializable
 
 log = get_logger(__file__)
 
+freshness_metrics = [
+    SimplePredefinedMetricName.HOURS_SINCE_LAST_LOAD,
+    SimplePredefinedMetricName.HOURS_SINCE_MAX_DATE,
+    SimplePredefinedMetricName.HOURS_SINCE_MAX_TIMESTAMP,
+    SimplePredefinedMetricName.FRESHNESS
+]
+
+volume_metrics = [
+    SimplePredefinedMetricName.VOLUME,
+    SimplePredefinedMetricName.ROWS_INSERTED,
+    SimplePredefinedMetricName.COUNT_ROWS
+]
+
+pipeline_reliability_metrics = [SimplePredefinedMetricName.COUNT_READ_QUERIES] + freshness_metrics + volume_metrics
+
+completeness_metrics = [
+    SimplePredefinedMetricName.PERCENT_NULL,
+    SimplePredefinedMetricName.COUNT_NULL,
+    SimplePredefinedMetricName.PERCENT_NOT_NULL,
+    SimplePredefinedMetricName.COUNT_NOT_NULL,
+    SimplePredefinedMetricName.PERCENT_EMPTY_STRING,
+    SimplePredefinedMetricName.COUNT_EMPTY_STRING,
+    SimplePredefinedMetricName.PERCENT_NAN,
+    SimplePredefinedMetricName.COUNT_NAN
+]
+
+uniqueness_metrics = [
+    SimplePredefinedMetricName.COUNT_DISTINCT,
+    SimplePredefinedMetricName.COUNT_DUPLICATES
+]
+
+distribution_metrics = [
+    SimplePredefinedMetricName.MAX,
+    SimplePredefinedMetricName.MIN,
+    SimplePredefinedMetricName.AVERAGE,
+    SimplePredefinedMetricName.VARIANCE,
+    SimplePredefinedMetricName.MEDIAN,
+    SimplePredefinedMetricName.SUM,
+    SimplePredefinedMetricName.SKEW,
+    SimplePredefinedMetricName.KURTOSIS,
+    SimplePredefinedMetricName.GEOMETRIC_MEAN,
+    SimplePredefinedMetricName.HARMONIC_MEAN,
+    SimplePredefinedMetricName.COUNT_FALSE,
+    SimplePredefinedMetricName.PERCENT_FALSE,
+    SimplePredefinedMetricName.COUNT_TRUE,
+    SimplePredefinedMetricName.PERCENT_TRUE
+]
+
+validity_metrics = [
+    SimplePredefinedMetricName.STRING_LENGTH_MAX,
+    SimplePredefinedMetricName.STRING_LENGTH_MIN,
+    SimplePredefinedMetricName.STRING_LENGTH_AVERAGE,
+    SimplePredefinedMetricName.PERCENT_UUID,
+    SimplePredefinedMetricName.COUNT_UUID,
+    SimplePredefinedMetricName.COUNT_CUSIP,
+    SimplePredefinedMetricName.PERCENT_CUSIP,
+    SimplePredefinedMetricName.COUNT_SEDOL,
+    SimplePredefinedMetricName.PERCENT_SEDOL,
+    SimplePredefinedMetricName.COUNT_ISIN,
+    SimplePredefinedMetricName.PERCENT_ISIN,
+    SimplePredefinedMetricName.COUNT_LEI,
+    SimplePredefinedMetricName.PERCENT_LEI,
+    SimplePredefinedMetricName.COUNT_FIGI,
+    SimplePredefinedMetricName.PERCENT_FIGI,
+    SimplePredefinedMetricName.COUNT_PERM_ID,
+    SimplePredefinedMetricName.PERCENT_PERM_ID,
+    SimplePredefinedMetricName.COUNT_NAN,
+    SimplePredefinedMetricName.PERCENT_NAN,
+    SimplePredefinedMetricName.COUNT_LONGITUDE,
+    SimplePredefinedMetricName.PERCENT_LONGITUDE,
+    SimplePredefinedMetricName.COUNT_LATITUDE,
+    SimplePredefinedMetricName.PERCENT_LATITUDE,
+    SimplePredefinedMetricName.COUNT_NOT_IN_FUTURE,
+    SimplePredefinedMetricName.PERCENT_NOT_IN_FUTURE,
+    SimplePredefinedMetricName.COUNT_DATE_NOT_IN_FUTURE,
+    SimplePredefinedMetricName.PERCENT_DATE_NOT_IN_FUTURE,
+    SimplePredefinedMetricName.COUNT_SSN,
+    SimplePredefinedMetricName.PERCENT_SSN,
+    SimplePredefinedMetricName.COUNT_EMAIL,
+    SimplePredefinedMetricName.PERCENT_EMAIL,
+    SimplePredefinedMetricName.COUNT_USA_PHONE,
+    SimplePredefinedMetricName.PERCENT_USA_PHONE,
+    SimplePredefinedMetricName.COUNT_USA_ZIP_CODE,
+    SimplePredefinedMetricName.PERCENT_USA_ZIP_CODE,
+    SimplePredefinedMetricName.PERCENT_UUID,
+    SimplePredefinedMetricName.COUNT_TIMESTAMP_STRING,
+    SimplePredefinedMetricName.PERCENT_TIMESTAMP_STRING,
+    SimplePredefinedMetricName.COUNT_USA_STATE_CODE,
+    SimplePredefinedMetricName.PERCENT_USA_STATE_CODE,
+    SimplePredefinedMetricName.PERCENT_VALUE_IN_LIST
+]
+
 
 def get_type_from_dict(obj: betterproto.Message) -> str:
     """A patch for beterproto.which_one_of not working on objects serialized from_dict.   I have reported the bug to
@@ -49,10 +142,87 @@ class SimpleMetricType(PydanticSubtypeSerializable, DatawatchFacade, ABC):
                                   SimplePredefinedMetricName.FRESHNESS]
         return [SimplePredefinedMetric(type="PREDEFINED", predefined_metric=i) for i in freshness_metric_names]
 
+    @classmethod
+    def get_volume_metric_types(cls) -> List[SimplePredefinedMetric]:
+        return cls.__get_simple_predefined_metrics(metric_names=volume_metrics)
+
+    @classmethod
+    def get_completeness_metric_types(cls) -> List[SimplePredefinedMetric]:
+        return cls.__get_simple_predefined_metrics(metric_names=completeness_metrics)
+
+    @classmethod
+    def get_uniqueness_metric_types(cls) -> List[SimplePredefinedMetric]:
+        return cls.__get_simple_predefined_metrics(metric_names=uniqueness_metrics)
+
+    @classmethod
+    def get_distribution_metric_types(cls) -> List[SimplePredefinedMetric]:
+        return cls.__get_simple_predefined_metrics(metric_names=distribution_metrics)
+
+    @classmethod
+    def get_validity_metric_types(cls) -> List[SimplePredefinedMetric]:
+        return cls.__get_simple_predefined_metrics(metric_names=validity_metrics)
+
+    @classmethod
+    def get_pipeline_reliability_metrics(cls) -> List[SimplePredefinedMetric]:
+        return cls.__get_simple_predefined_metrics(metric_names=pipeline_reliability_metrics)
+
+    @classmethod
+    def __get_simple_predefined_metrics(cls, metric_names: List[SimplePredefinedMetricName]):
+        return [SimplePredefinedMetric(type="PREDEFINED", predefined_metric=i) for i in metric_names]
+
     @classmethod
     def is_freshness_metric(cls, predefined_metric):
         return predefined_metric in cls.get_freshness_metric_types()
 
+    @classmethod
+    def is_volume_metric(cls, predefined_metric):
+        return predefined_metric in cls.get_volume_metric_types()
+
+    @classmethod
+    def is_pipeline_reliability(cls, predefined_metric):
+        return predefined_metric in cls.get_pipeline_reliability_metrics()
+
+    @classmethod
+    def is_completeness_metric(cls, predefined_metric):
+        return predefined_metric in cls.get_completeness_metric_types()
+
+    @classmethod
+    def is_uniqueness_metric(cls, predefined_metric):
+        return predefined_metric in cls.get_uniqueness_metric_types()
+
+    @classmethod
+    def is_distribution_metric(cls, predefined_metric):
+        return predefined_metric in cls.get_distribution_metric_types()
+
+    @classmethod
+    def is_validity_metric(cls, predefined_metric):
+        return predefined_metric in cls.get_validity_metric_types()
+
+    @classmethod
+    def is_freshness_volume(cls, predefined_metric):
+        return predefined_metric in [
+            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.FRESHNESS),
+            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.VOLUME)
+        ]
+
+    @classmethod
+    def is_table_metric(cls, predefined_metric):
+        return predefined_metric in [
+            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.FRESHNESS),
+            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.VOLUME),
+            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.COUNT_ROWS),
+            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.COUNT_READ_QUERIES),
+        ]
+
+    @classmethod
+    def get_metadata_metrics(cls) -> List[SimplePredefinedMetric]:
+        metadata_metrics = [SimplePredefinedMetricName.HOURS_SINCE_LAST_LOAD,
+                            SimplePredefinedMetricName.FRESHNESS,
+                            SimplePredefinedMetricName.VOLUME,
+                            SimplePredefinedMetricName.ROWS_INSERTED,
+                            SimplePredefinedMetricName.COUNT_READ_QUERIES]
+        return [SimplePredefinedMetric(type="PREDEFINED", predefined_metric=i) for i in metadata_metrics]
+
     @classmethod
     def from_datawatch_object(cls, obj: MetricType) -> SimpleMetricType:
         t = betterproto.which_one_of(obj, "metric_type")[0]
@@ -202,7 +372,12 @@ class SimpleMetricSchedule(YamlModelWithValidatorContext, DatawatchFacade):
     def from_datawatch_object(cls, obj: MetricSchedule) -> Optional[SimpleMetricSchedule]:
         if obj.schedule_frequency:
             if obj.schedule_frequency.interval_type == 0 and obj.schedule_frequency.interval_value == 0:
-                return None
+                if obj.named_schedule:
+                    return SimpleMetricSchedule(
+                        named_schedule=SimpleNamedSchedule.from_datawatch_object(obj.named_schedule)
+                    )
+                else:
+                    return None
             else:
                 sti = SimpleTimeInterval(
                     interval_type=SimpleTimeIntervalType.from_datawatch_object(
@@ -243,9 +418,18 @@ class SimpleThreshold(PydanticSubtypeSerializable, DatawatchFacade, ABC):
         if not t:
             t = get_type_from_dict(obj[0])
         if t == 'auto_threshold':
+            lower_bound = None
+            upper_bound = None
+            for i in obj:
+                if i.auto_threshold.bound.bound_type == SimpleBoundType.UPPER_BOUND_SIMPLE_BOUND_TYPE:
+                    upper_bound = i.auto_threshold.bound.value
+                if i.auto_threshold.bound.bound_type == SimpleBoundType.LOWER_BOUND_SIMPLE_BOUND_TYPE:
+                    lower_bound = i.auto_threshold.bound.value
             sat = SimpleAutoThreshold(
                 type="AUTO",
-                sensitivity=SimpleAutothresholdSensitivity.from_datawatch_object(obj[0].auto_threshold.sensitivity)
+                sensitivity=SimpleAutothresholdSensitivity.from_datawatch_object(obj[0].auto_threshold.sensitivity),
+                upper_bound_only=True if upper_bound and not lower_bound else False,
+                lower_bound_only=True if lower_bound and not upper_bound else False
             )
             return sat
         elif t == 'constant_threshold':
@@ -299,6 +483,8 @@ class SimpleThreshold(PydanticSubtypeSerializable, DatawatchFacade, ABC):
                 type='FRESHNESS',
                 cron=t.cron, timezone=t.timezone, upper_bound=bound, delay_at_update=dau)
             return ft
+        elif t == 'none_threshold':
+            return SimpleNoneThreshold(type="NONE")
         else:
             error_message = f"Threshold type not supported: {t}"
             raise InvalidConfigurationException(error_message)
@@ -307,6 +493,19 @@ class SimpleThreshold(PydanticSubtypeSerializable, DatawatchFacade, ABC):
 class SimpleAutoThreshold(SimpleThreshold, type='AUTO'):
     sensitivity: SimpleAutothresholdSensitivity = SimpleAutothresholdSensitivity.MEDIUM
     upper_bound_only: bool = False
+    lower_bound_only: bool = False
+
+    @root_validator(pre=True)
+    def check_bounds(cls, values):
+        if values.get('upper_bound_only') and values.get('lower_bound_only'):
+            errlns = [
+                yaml.safe_dump({"upper_bound_only": True}, indent=True, sort_keys=False),
+                yaml.safe_dump({"lower_bound_only": True}, indent=True, sort_keys=False),
+            ]
+            cls.register_validation_error(error_lines=errlns, error_message=INVALID_THRESHOLD_BOUNDS)
+
+        return values
+
 
     def to_datawatch_object(self, **kwargs) -> List[Threshold]:
         lb = SimpleBound(bound_type=SimpleBoundType.LOWER_BOUND_SIMPLE_BOUND_TYPE, value=-1.0)
@@ -329,6 +528,8 @@ class SimpleAutoThreshold(SimpleThreshold, type='AUTO'):
         )
         if self.upper_bound_only:
             return [Threshold(auto_threshold=ubat)]
+        elif self.lower_bound_only:
+            return [Threshold(auto_threshold=lbat)]
         else:
             return [Threshold(auto_threshold=lbat), Threshold(auto_threshold=ubat)]
 
@@ -373,9 +574,25 @@ class SimpleFreshnessThreshold(SimpleThreshold, type='FRESHNESS'):
 
 
 class SimpleRelativeThreshold(SimpleThreshold, type='RELATIVE'):
-    lookback: SimpleTimeInterval
+    lookback: SimpleTimeInterval = None
     upper_bound: Optional[float] = None
     lower_bound: Optional[float] = None
+    reference_point: SimpleTimeInterval = None
+
+    @root_validator(pre=True)
+    def validate_lookback_or_reference_point_set(cls, values):
+        # TODO use field aliases after upgrading pydantic version
+        #  https://docs.pydantic.dev/2.3/usage/fields/#aliaspath-and-aliaschoices
+        lookback = values.get('lookback')
+        reference_point = values.get('reference_point')
+
+        if reference_point:
+            values["lookback"] = reference_point
+
+        if not lookback and not reference_point:
+            raise ValueError('For relative thresholds, reference_point is required.')
+
+        return values
 
     def to_datawatch_object(self, **kwargs) -> List[Threshold]:
         lkbk = self.lookback.to_datawatch_object()
@@ -394,9 +611,25 @@ class SimpleRelativeThreshold(SimpleThreshold, type='RELATIVE'):
 
 
 class SimpleStdDevThreshold(SimpleThreshold, type='STDDEV'):
-    lookback: SimpleTimeInterval
+    lookback: SimpleTimeInterval = None
     upper_bound: Optional[float] = None
     lower_bound: Optional[float] = None
+    reference_point: SimpleTimeInterval = None
+
+    @root_validator(pre=True)
+    def validate_lookback_or_reference_point_set(cls, values):
+        # TODO use field aliases after upgrading pydantic version
+        #  https://docs.pydantic.dev/2.3/usage/fields/#aliaspath-and-aliaschoices
+        lookback = values.get('lookback')
+        reference_point = values.get('reference_point')
+
+        if reference_point:
+            values["lookback"] = reference_point
+
+        if not lookback and not reference_point:
+            raise ValueError('For standard deviation thresholds, reference_point is required.')
+
+        return values
 
     def to_datawatch_object(self, **kwargs) -> List[Threshold]:
         lkbk = self.lookback.to_datawatch_object()
@@ -414,6 +647,12 @@ class SimpleStdDevThreshold(SimpleThreshold, type='STDDEV'):
         return thresholds
 
 
+class SimpleNoneThreshold(SimpleThreshold, type='NONE'):
+
+    def to_datawatch_object(self, **kwargs) -> Threshold:
+        return Threshold(none_threshold=NoneThreshold())
+
+
 SNC = TypeVar('SNC', bound='SimpleNotificationChannel')
 
 
@@ -434,8 +673,8 @@ class SimpleNotificationChannel(PydanticSubtypeSerializable, DatawatchFacade, AB
                 thread_ts=obj.slack_channel_info.thread_ts
             )
         elif t == 'webhook':
-            key: str
-            value: str
+            key: Optional[str] = None
+            value: Optional[str] = None
             if obj.webhook.webhook_headers:
                 key = obj.webhook.webhook_headers[0].key
                 value = obj.webhook.webhook_headers[0].value
@@ -618,7 +857,7 @@ class SimpleCollection(YamlModelWithValidatorContext, DatawatchFacade):
 
         return values
 
-    def merge_for_upsert(self, existing: SimpleCollection, overwrite: bool = True) -> SimpleCollection:
+    def merge_for_upsert(self, existing: SimpleCollection) -> SimpleCollection:
         """
         Merges this collection and existing collection.  Attributes that will be merged include: notification channels and metric ids.
 
@@ -639,26 +878,20 @@ class SimpleCollection(YamlModelWithValidatorContext, DatawatchFacade):
         # set existing SLA ID.
         new_collection.id = existing.id
 
-        if not overwrite:
-            """If we are not overwriting then we want to merge notification channels and metric ids."""
-            if not new_collection.name:
-                new_collection.name = existing.name
+        """We match on name. If users want to edit description or the muted_until_timestamp values."""
+        if not new_collection.description:
+            new_collection.description = existing.description
 
-            if not new_collection.description:
-                new_collection.description = existing.description
+        if not new_collection.muted_until_timestamp:
+            new_collection.muted_until_timestamp = existing.muted_until_timestamp
 
-            if not new_collection.muted_until_timestamp:
-                new_collection.muted_until_timestamp = existing.muted_until_timestamp
+        """Notification channels are missing because those will always be controlled by incoming bigconfig."""
 
-            if new_collection.notification_channels:
-                new_collection.notification_channels.extend(existing.notification_channels)
-            else:
-                new_collection.notification_channels = existing.notification_channels
-
-            if new_collection.metric_ids:
-                new_collection.metric_ids.extend(existing.metric_ids)
-            else:
-                new_collection.metric_ids = existing.metric_ids
+        """This will ensure that the collection will retain metrics when the above fields are updated."""
+        if new_collection.metric_ids:
+            new_collection.metric_ids.extend(existing.metric_ids)
+        else:
+            new_collection.metric_ids = existing.metric_ids
 
         return new_collection
 
@@ -732,10 +965,13 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
     notification_channels: Optional[List[SimpleNotificationChannel]] = None
     parameters: Optional[List[SimpleMetricParameter]] = None
     lookback: Optional[SimpleLookback] = None
+    grain_seconds: Optional[int] = 0
     muted_until_epoch_seconds: Optional[int] = 0
     sla_ids: Optional[List[int]] = Field(default_factory=lambda: [])
     collection_ids: Optional[List[int]] = Field(default_factory=lambda: [])
     metric_schedule: Optional[SimpleMetricSchedule] = None
+    user_favorites: Optional[List[str]] = None
+    rct_overrides: Optional[List[str]] = None
 
     @validator('threshold', always=True, pre=False)
     def set_default_threshold(cls, threshold, values):
@@ -786,6 +1022,13 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
                 interval_type=SimpleTimeIntervalType.HOURS, interval_value=6
             )
             return values
+        elif metric_type \
+                and SimpleMetricType.is_freshness_volume(metric_type):
+            """Freshness and volume should execute every 6 hours by default"""
+            values['schedule_frequency'] = SimpleTimeInterval(
+                interval_type=SimpleTimeIntervalType.HOURS, interval_value=6
+            )
+            return values
         else:
             """All other metrics should execute every 24 hours by default WIZ-1623"""
             values['schedule_frequency'] = SimpleTimeInterval(
@@ -809,7 +1052,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
 
     @validator('metric_type', pre=True)
     def must_be_dict_type(cls, metric_type):
-        if isinstance(metric_type, SimplePredefinedMetric):
+        if isinstance(metric_type, (SimplePredefinedMetric, SimpleTemplateMetric)):
             return metric_type
 
         if metric_type and not isinstance(metric_type, dict):
@@ -866,6 +1109,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
         """
         self.deployments_must_have_metric_type_if_not_saved_metric_id(config_error_lines=config_error_lines)
         self.deployments_cannot_have_metric_type_if_has_saved_metric_id(config_error_lines=config_error_lines)
+        self.deployments_freshness_metrics_must_have_valid_lookback_type(config_error_lines=config_error_lines)
 
     def deployments_must_have_metric_type_if_not_saved_metric_id(self, config_error_lines: List[str]):
         """
@@ -896,11 +1140,38 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
                                            error_context_lines=config_error_lines,
                                            error_message=error_message)
 
+    def deployments_freshness_metrics_must_have_valid_lookback_type(self, config_error_lines: List[str]):
+        """
+        Freshness metrics must have a data time window type. If any other window type is chosen then log
+        a warning message. The valid lookback type is automatically applied to the  metric in the backend, 
+        but user should be informed of this to limit confusion. 
+
+        See https://linear.app/torodata/issue/WIZ-2583/[061223]-metric-deployed-via-bigconfig-not-showing-as-clock-time
+
+        Args:
+            config_error_lines: The configuration serialized to yaml and split to lines.
+
+        Returns: None
+        """
+        if SimpleMetricType.is_freshness_metric(self.metric_type) \
+                and self.lookback \
+                and self.lookback.lookback_type != SimpleLookbackType.DATA_TIME:
+            if self.saved_metric_id:
+                log.warning(
+                    f'Invalid freshness metric configuration. Saved metric id: {self.saved_metric_id} was given a '
+                    f'lookback of type {self.lookback.lookback_type.name}. This will be changed to '
+                    f'type {SimpleLookbackType.DATA_TIME}')
+            else:
+                log.warning(
+                    f'Invalid freshness metric configuration. Metric: {self.metric_type} was given a lookback of '
+                    f'type {self.lookback.lookback_type.name}. This will be changed to '
+                    f'type {SimpleLookbackType.DATA_TIME}')
+
     def saved_metrics_must_have_id(self, config_error_lines: List[str]):
         """
         Saved Metric Definitions must have a saved metric id.
         Args:
-            config_error_lines: The congiruation serialized to yaml and split to lines.
+            config_error_lines: The configuration serialized to yaml and split to lines.
 
         Returns: None
         """
@@ -929,18 +1200,31 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
 
         builder.parameters = [SimpleMetricParameter.from_datawatch_object(p) for p in obj.parameters]
 
-        builder.lookback = SimpleLookback(lookback_window=SimpleTimeInterval.from_datawatch_object(obj.lookback),
-                                          lookback_type=SimpleLookbackType.from_datawatch_object(obj.lookback_type),
-                                          bucket_size=BucketSize.from_seconds(obj.grain_seconds))
+        if obj.lookback_type != LookbackType.UNDEFINED_LOOKBACK_TYPE:
+            builder.lookback = SimpleLookback(lookback_window=SimpleTimeInterval.from_datawatch_object(obj.lookback),
+                                              lookback_type=SimpleLookbackType.from_datawatch_object(obj.lookback_type),
+                                              bucket_size=BucketSize.from_seconds(obj.grain_seconds))
 
         builder.muted_until_epoch_seconds = obj.muted_until_epoch_seconds
+        builder.grain_seconds = obj.grain_seconds
+
+        if isinstance(obj, MetricDefinition):
+            builder.collection_ids = obj.collection_ids
 
-        builder.collection_ids = obj.collection_ids
         builder.metric_schedule = SimpleMetricSchedule.from_datawatch_object(obj.metric_schedule)
+
+        if isinstance(obj, MetricDefinition):
+            builder.rct_overrides = obj.rct_overrides
+        if isinstance(obj, MetricConfiguration):
+            builder.rct_overrides = obj.rct_override.split(",") if obj.rct_override else None
+
         return builder
 
-    def to_datawatch_object(self, **kwargs) -> MetricDefinition:
-        builder = MetricDefinition()
+    def to_datawatch_object(self, to_config: bool = False, **kwargs) -> Union[MetricDefinition, MetricConfiguration]:
+        if to_config:
+            builder = MetricConfiguration()
+        else:
+            builder = MetricDefinition()
 
         # Verifying that metric_type has been set before serializing to datawatch object.
         if self.metric_type:
@@ -974,12 +1258,11 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
             if self.lookback.lookback_type:
                 builder.lookback_type = self.lookback.lookback_type.to_datawatch_object()
 
-            """Freshness metrics require data time lookback"""
-            if SimpleMetricType.is_freshness_metric(self.metric_type):
-                builder.lookback_type = SimpleLookbackType.DATA_TIME.to_datawatch_object()
-
             builder.grain_seconds = self.lookback.bucket_size.to_seconds()
 
+        if SimpleMetricType.is_freshness_volume(self.metric_type):
+            builder.grain_seconds = BucketSize.HOUR.to_seconds()
+
         builder.muted_until_epoch_seconds = self.muted_until_epoch_seconds
 
         if self.collection_ids:
@@ -987,4 +1270,13 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
         if self.metric_schedule:
             builder.metric_schedule = self.metric_schedule.to_datawatch_object()
 
+        if self.user_favorites:
+            builder.tags.extend(self.user_favorites)
+
+        if self.rct_overrides:
+            if len(self.rct_overrides) == 1 and self.rct_overrides[0] == "FULL_SCAN":
+                builder.rct_overrides = ["bigeye-no-rct"]
+            else:
+                builder.rct_overrides = self.rct_overrides
+
         return builder
